﻿Public Class Form1

    Dim serialport As New System.IO.Ports.SerialPort    ' Declares a handle for COM port
    Dim serBuf(512) As Byte                              ' A global accessable array for data to be buffered for the COMX port
    Dim servonumber As Integer = 0

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ' Show all available COM ports.
        For Each sp As String In My.Computer.Ports.SerialPortNames
            ComboBox_comport().Items.Add(sp)            ' Adds COMX to listbox called "comportselect"
        Next
        ComboBox_comport.Text = "Select COM Port"       ' Adds the message when program is first loaded

        ComboBox_servo.Items.Add("Select Servo")
        For x As Integer = 1 To 84                      ' Fill the servo selection box with the servo numbers
            ComboBox_servo.Items.Add(x)
        Next x
    End Sub

    Private Sub ComboBox_comport_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBox_comport.SelectedIndexChanged
        If (serialport.IsOpen) Then                                                                 ' If program has previously opened a COM port, start by closing it
            serialport.Close()
        End If
        Try
            serialport = My.Computer.Ports.OpenSerialPort(ComboBox_comport.Text, 115200, 0, 8, 2)    ' attempt to open a new COM port
        Catch                                                                                       ' Catch open failures
            MessageBox.Show("Failed to open COM port", "SD84", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk)
            Return
        End Try
        serialport.ReadTimeout = 1000                                                               ' We will only arrive here on successfull open

        ' Set all channels to servo mode
        serBuf(0) = &HAA
        serBuf(1) = &HA0
        serBuf(2) = &H55
        serBuf(3) = &H4                 ' Set mode command
        serBuf(4) = &H1                 ' Start at channel 1
        serBuf(5) = &H54                ' 84 mode bytes to follow
        For x As Integer = 6 To 90      ' Fill serbuf with servo mode bytes
            serBuf(x) = &H19
        Next x
        transmit(90)                    ' Send data 
        receive(1)                      ' get responce
    End Sub

    Private Sub servoSelected(sender As Object, e As EventArgs) Handles ComboBox_servo.SelectedIndexChanged
        If TypeOf sender Is ComboBox Then
            Dim combo = DirectCast(sender, ComboBox)
            servonumber = combo.SelectedIndex       ' The index of the selected item is the servo number
        End If
    End Sub

    Private Sub servoPosSet(sender As Object, e As EventArgs) Handles TrackBar_pos.ValueChanged
        If TypeOf sender Is TrackBar Then
            Dim track = DirectCast(sender, TrackBar)
            Dim pos As Integer = track.Value
            Dim high As Byte = (pos And &HFF00) >> 8
            Dim low As Byte = pos And &HFF
            serBuf(0) = &HAA
            serBuf(1) = &HA0
            serBuf(2) = &H55
            serBuf(3) = &H1             ' Set servo command
            serBuf(4) = servonumber     ' Servo we wish to set
            serBuf(5) = &H2             ' 2 Data bytes coming
            serBuf(6) = low             ' Low byte of position
            serBuf(7) = high            ' High byte of position
            transmit(8)
            receive(1)
        End If
    End Sub

    Private Sub transmit(num As Integer)
        If serialport.IsOpen Then
            Try
                serialport.Write(serBuf, 0, num)            ' writes specified amount of SerBuf out on COM port
            Catch
                serialport.Close()
                MessageBox.Show("Failed to read from COM port", "ERROR!", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk)
                Return
            End Try
        End If
    End Sub

    Private Sub receive(num As Integer)
        If serialport.IsOpen Then
            Dim x As Integer = 0
            For x = 0 To num - 1                            ' Call the read function for passed number of times
                Try                                         ' This way we receive the correct number of bytes while still using timeouts
                    serialport.Read(serBuf, x, 1)
                Catch ex As Exception
                    serialport.Close()
                    MessageBox.Show("Failed to read from COM port", "ERROR!", MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk)
                End Try
            Next x
        End If
    End Sub
End Class
